#
# Copyright (c) 2008 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
# 
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation. 
#

use strict;

package RHN::Kickstart;

use Params::Validate qw/:all/;
Params::Validate::validation_options(strip_leading => "-");

use RHN::DB::Kickstart;
our @ISA = qw/RHN::DB::Kickstart/;
use RHN::Kickstart::Commands;
use RHN::Kickstart::Packages;

use RHN::SessionSwap;

use PXT::Utils;

our @simple_struct_fields = @RHN::DB::Kickstart::simple_struct_fields;

sub render {
  my $self = shift;

  my $text = shift || <<EOQ;
# Kickstart config file generated by RHN Config Management
#
# Profile Name: {kickstart_name}
# Profile Label: {kickstart_label}
#
{kickstart_commands}

%packages {kickstart_package_options}

{kickstart_packages}

%pre
{kickstart_pre}

{kickstart_interpreter_pre}

%post --nochroot
{kickstart_nochroot_post}

{kickstart_interpreter_post}

%post
{kickstart_post}
# end of generated kickstart file
EOQ

  my %subs;

  $subs{"kickstart_${_}"} = $self->$_ foreach qw/name label pre post nochroot_post/;
  $subs{kickstart_commands} = $self->commands ? $self->commands->render : '# No commands section defined for kickstart profile';
  $subs{kickstart_package_options} = $self->render_package_options || '';
  $subs{kickstart_packages} = $self->packages->render || '';
  $subs{kickstart_interpreter_post} = $self->render_interpreter_post || '';
  $subs{kickstart_interpreter_pre} = $self->render_interpreter_pre || '';

  my $ret = PXT::Utils->perform_substitutions($text, \%subs);

  return $ret;
}

sub render_package_options {
  my $self = shift;

  my $options = join(" ", map { "--${_}" } @{$self->package_options}) || '';

  return $options;
}

sub render_interpreter_post {
  my $self = shift;
  my $post = '';
  $post = '%post --interpreter ' 
			.  $self->interpreter_post_val 
			. "\n" . $self->interpreter_post_script if $self->interpreter_post_val;
  return $post; 
}

sub render_interpreter_pre {
  my $self = shift;
  my $pre = '';
  $pre = '%pre --interpreter ' 
			. $self->interpreter_pre_val 
			. "\n" . $self->interpreter_pre_script if $self->interpreter_pre_val;
  return $pre; 
}

sub set_zerombr {
  my $self = shift;
  my $on = shift;

  return unless $self->commands;

  if (defined $on and $on ne 'no') {
    $self->commands->set(zerombr => 'yes');
  }
  else {
    $self->commands->unset('zerombr');
  }

  return;
}

sub set_bootloader {
  my $self = shift;
  my %params = validate(@_, { bootloader => 1, append => 0, driveorder => 0, location => 0,
			      password => 0, md5pass => 0, upgrade => 0 });

  return unless $self->commands;

  my $location = $params{location} ? $params{location} : 'mbr';
  $location = 'mbr' unless (grep { $location eq $_ } qw/mbr partition none/);
  $location = '--location ' . $location;

  my $uselilo = ($params{bootloader} eq 'lilo') ? '--useLilo ' : '';

  my $extra;

  foreach my $key (qw/append driveorder password md5pass upgrade/) {
    if ($params{$key}) {
      $extra .= "--$key $params{$key}";
    }
    elsif (exists $params{$key}) {
      $extra .= "--$key";
    }
  }

  $extra = " " . $extra if $extra;

  $self->commands->bootloader("${uselilo}${location}${extra}");

  return;
}

sub get_bootloader {
  my $self = shift;

  return unless $self->commands;

  my $bl = $self->commands->bootloader;
  my @opts = @{$bl};
  return (grep { $_ =~ /lilo/i } @opts) ? 'lilo' : 'grub';
}

sub set_timezone {
  my $self = shift;
  my %params = validate(@_, { zone => 1, hardware_utc => 0 });

  return unless $self->commands;

  $params{zone} = 'America/New_York' unless (grep { $params{zone} eq $_ } $self->timezones);
  my $utc = $params{hardware_utc} ? '--utc ' : '';

  $self->commands->timezone($utc . $params{zone});

  return;
}

sub get_timezone {
  my $self = shift;

  return unless $self->commands;

  my $zn = $self->commands->timezone || '';

  my @opts = @{$zn};
  $zn = $opts[-1];

  return ( grep { $zn eq $_ } $self->timezones ) ? $zn : 'America/New_York';
}

sub is_timezone_utc {
  my $self = shift;

  return unless $self->commands;

  my $zn = $self->commands->timezone || '';

  foreach my $opt (@{$zn}) {
    return 1 if ($opt eq '--utc');
  }

  return;
}

sub set_partition_info {
  my $self = shift;
  my %params = validate(@_, { parts => 1 });

  return unless $self->commands;
  my (@parts, @raids, @volgroups, @logvols, @include);
  foreach my $line (split /\n/, $params{parts}) {
    if ($line =~ /^part\S*\s+(.*)$/) {
      my $part = $1;
      push @parts, [ split /\s+/, $part ];
    }
    elsif ($line =~ /^raid\s+(.*)$/) {
      my $raid = $1;
      push @raids, [ split /\s+/, $raid ];
    }
    elsif ($line =~ /^volgroup\s+(.*)$/) {
      my $group = $1;
      push @volgroups, [ split /\s+/, $group ];
    }
    elsif ($line =~ /^logvol\s+(.*)$/) {
      my $vol = $1;
      push @logvols, [ split /\s+/, $vol ];
    }
    elsif ($line =~ /^\%?include\s+(.*)$/) {
      my $inc = $1;
      push @include, [ split /\s+/, $inc];
    }
  }

  $self->commands->partitions(new RHN::Kickstart::Partitions(@parts));
  $self->commands->raids(new RHN::Kickstart::Raids(@raids));
  $self->commands->volgroups(new RHN::Kickstart::Volgroups(@volgroups));
  $self->commands->logvols(new RHN::Kickstart::Logvols(@logvols));
  $self->commands->include(new RHN::Kickstart::Include(@include));

  return;
}

my @timezones = qw|Europe/Andorra
Asia/Dubai
Asia/Kabul
America/Antigua
America/Anguilla
Europe/Tirane
Asia/Yerevan
America/Curacao
Africa/Luanda
Antarctica/McMurdo
Antarctica/South_Pole
Antarctica/Palmer
Antarctica/Mawson
Antarctica/Davis
Antarctica/Casey
Antarctica/Vostok
Antarctica/DumontDUrville
Antarctica/Syowa
America/Buenos_Aires
America/Cordoba
America/Jujuy
America/Catamarca
America/Mendoza
Pacific/Pago_Pago
Europe/Vienna
Australia/Lord_Howe
Australia/Hobart
Australia/Melbourne
Australia/Sydney
Australia/Broken_Hill
Australia/Brisbane
Australia/Lindeman
Australia/Adelaide
Australia/Darwin
Australia/Perth
America/Aruba
Asia/Baku
Europe/Sarajevo
America/Barbados
Asia/Dhaka
Europe/Brussels
Africa/Ouagadougou
Europe/Sofia
Asia/Bahrain
Africa/Bujumbura
Africa/Porto-Novo
Atlantic/Bermuda
Asia/Brunei
America/La_Paz
America/Noronha
America/Belem
America/Fortaleza
America/Recife
America/Araguaina
America/Maceio
America/Sao_Paulo
America/Cuiaba
America/Porto_Velho
America/Boa_Vista
America/Manaus
America/Eirunepe
America/Rio_Branco
America/Nassau
Asia/Thimphu
Africa/Gaborone
Europe/Minsk
America/Belize
America/St_Johns
America/Halifax
America/Glace_Bay
America/Goose_Bay
America/Montreal
America/Nipigon
America/Thunder_Bay
America/Pangnirtung
America/Iqaluit
America/Rankin_Inlet
America/Winnipeg
America/Rainy_River
America/Cambridge_Bay
America/Regina
America/Swift_Current
America/Edmonton
America/Yellowknife
America/Inuvik
America/Dawson_Creek
America/Vancouver
America/Whitehorse
America/Dawson
Indian/Cocos
Africa/Kinshasa
Africa/Lubumbashi
Africa/Bangui
Africa/Brazzaville
Europe/Zurich
Africa/Abidjan
Pacific/Rarotonga
America/Santiago
Pacific/Easter
Africa/Douala
Asia/Shanghai
Asia/Harbin
Asia/Chongqing
Asia/Urumqi
Asia/Kashgar
America/Bogota
America/Costa_Rica
America/Havana
Atlantic/Cape_Verde
Indian/Christmas
Asia/Nicosia
Europe/Prague
Europe/Berlin
Africa/Djibouti
Europe/Copenhagen
America/Dominica
America/Santo_Domingo
Africa/Algiers
America/Guayaquil
Pacific/Galapagos
Europe/Tallinn
Africa/Cairo
Africa/El_Aaiun
Africa/Asmera
Europe/Madrid
Africa/Ceuta
Atlantic/Canary
Africa/Addis_Ababa
Europe/Helsinki
Pacific/Fiji
Atlantic/Stanley
Pacific/Yap
Pacific/Truk
Pacific/Ponape
Pacific/Kosrae
Atlantic/Faeroe
Europe/Paris
Africa/Libreville
Europe/London
Europe/Belfast
America/Grenada
Asia/Tbilisi
America/Cayenne
Africa/Accra
Europe/Gibraltar
America/Godthab
America/Danmarkshavn
America/Scoresbysund
America/Thule
Africa/Banjul
Africa/Conakry
America/Guadeloupe
Africa/Malabo
Europe/Athens
Atlantic/South_Georgia
America/Guatemala
Pacific/Guam
Africa/Bissau
America/Guyana
Asia/Hong_Kong
America/Tegucigalpa
Europe/Zagreb
America/Port-au-Prince
Europe/Budapest
Asia/Jakarta
Asia/Pontianak
Asia/Makassar
Asia/Jayapura
Europe/Dublin
Asia/Jerusalem
Asia/Calcutta
Indian/Chagos
Asia/Baghdad
Asia/Tehran
Atlantic/Reykjavik
Europe/Rome
America/Jamaica
Asia/Amman
Asia/Tokyo
Africa/Nairobi
Asia/Bishkek
Asia/Phnom_Penh
Pacific/Tarawa
Pacific/Enderbury
Pacific/Kiritimati
Indian/Comoro
America/St_Kitts
Asia/Pyongyang
Asia/Seoul
Asia/Kuwait
America/Cayman
Asia/Almaty
Asia/Qyzylorda
Asia/Aqtobe
Asia/Aqtau
Asia/Oral
Asia/Vientiane
Asia/Beirut
America/St_Lucia
Europe/Vaduz
Asia/Colombo
Africa/Monrovia
Africa/Maseru
Europe/Vilnius
Europe/Luxembourg
Europe/Riga
Africa/Tripoli
Africa/Casablanca
Europe/Monaco
Europe/Chisinau
Indian/Antananarivo
Pacific/Majuro
Pacific/Kwajalein
Europe/Skopje
Africa/Bamako
Africa/Timbuktu
Asia/Rangoon
Asia/Ulaanbaatar
Asia/Hovd
Asia/Choibalsan
Asia/Macau
Pacific/Saipan
America/Martinique
Africa/Nouakchott
America/Montserrat
Europe/Malta
Indian/Mauritius
Indian/Maldives
Africa/Blantyre
America/Mexico_City
America/Cancun
America/Merida
America/Monterrey
America/Mazatlan
America/Chihuahua
America/Hermosillo
America/Tijuana
Asia/Kuala_Lumpur
Asia/Kuching
Africa/Maputo
Africa/Windhoek
Pacific/Noumea
Africa/Niamey
Pacific/Norfolk
Africa/Lagos
America/Managua
Europe/Amsterdam
Europe/Oslo
Asia/Katmandu
Pacific/Nauru
Pacific/Niue
Pacific/Auckland
Pacific/Chatham
Asia/Muscat
America/Panama
America/Lima
Pacific/Tahiti
Pacific/Marquesas
Pacific/Gambier
Pacific/Port_Moresby
Asia/Manila
Asia/Karachi
Europe/Warsaw
America/Miquelon
Pacific/Pitcairn
America/Puerto_Rico
Asia/Gaza
Europe/Lisbon
Atlantic/Madeira
Atlantic/Azores
Pacific/Palau
America/Asuncion
Asia/Qatar
Indian/Reunion
Europe/Bucharest
Europe/Kaliningrad
Europe/Moscow
Europe/Samara
Asia/Yekaterinburg
Asia/Omsk
Asia/Novosibirsk
Asia/Krasnoyarsk
Asia/Irkutsk
Asia/Yakutsk
Asia/Vladivostok
Asia/Sakhalin
Asia/Magadan
Asia/Kamchatka
Asia/Anadyr
Africa/Kigali
Asia/Riyadh
Pacific/Guadalcanal
Indian/Mahe
Africa/Khartoum
Europe/Stockholm
Asia/Singapore
Atlantic/St_Helena
Europe/Ljubljana
Arctic/Longyearbyen
Atlantic/Jan_Mayen
Europe/Bratislava
Africa/Freetown
Europe/San_Marino
Africa/Dakar
Africa/Mogadishu
America/Paramaribo
Africa/Sao_Tome
America/El_Salvador
Asia/Damascus
Africa/Mbabane
America/Grand_Turk
Africa/Ndjamena
Indian/Kerguelen
Africa/Lome
Asia/Bangkok
Asia/Dushanbe
Pacific/Fakaofo
Asia/Ashgabat
Africa/Tunis
Pacific/Tongatapu
Asia/Dili
Europe/Istanbul
America/Port_of_Spain
Pacific/Funafuti
Asia/Taipei
Africa/Dar_es_Salaam
Europe/Kiev
Europe/Uzhgorod
Europe/Zaporozhye
Europe/Simferopol
Africa/Kampala
Pacific/Johnston
Pacific/Midway
Pacific/Wake
America/New_York
America/Detroit
America/Louisville
America/Kentucky/Monticello
America/Indianapolis
America/Indiana/Marengo
America/Indiana/Knox
America/Indiana/Vevay
America/Chicago
America/Menominee
America/North_Dakota/Center
America/Denver
America/Boise
America/Shiprock
America/Phoenix
America/Los_Angeles
America/Anchorage
America/Juneau
America/Yakutat
America/Nome
America/Adak
Pacific/Honolulu
America/Montevideo
Asia/Samarkand
Asia/Tashkent
Europe/Vatican
America/St_Vincent
America/Caracas
America/Tortola
America/St_Thomas
Asia/Saigon
Pacific/Efate
Pacific/Wallis
Pacific/Apia
Asia/Aden
Indian/Mayotte
Europe/Belgrade
Africa/Johannesburg
Africa/Lusaka
Africa/Harare|;

sub timezones {
  return @timezones;
}

1;
